! THIS VERSION: 21/09/2011 AT 15:15:00 GMT.

!-*-*-*-*-*-  G A L A H A D  -  D U M M Y   M A 9 7    M O D U L E  -*-*-*-*-*-

module hsl_MA97_double
    
   USE GALAHAD_SYMBOLS

! Parameters (all private)
  integer, parameter, private  :: short = kind(0)
  integer(short), parameter, private  :: wp = kind(0.0d0)
  integer(short), parameter, private  :: long = selected_int_kind(18)
  real(wp), parameter, private :: one = 1.0_wp
  real(wp), parameter, private :: zero = 0.0_wp
  integer(short), parameter, private :: nemin_default = 8

  interface MA97_analyse
      module procedure MA97_analyse_double
  end interface

  interface MA97_analyse_coord
      module procedure MA97_analyse_coord_double
  end interface

  interface MA97_factor
      module procedure MA97_factor_double
  end interface

  interface MA97_factor_solve
      module procedure MA97_factor_solve_double
      module procedure MA97_factor_solve_one_double
  end interface

  interface MA97_solve
      module procedure MA97_solve_double
      module procedure MA97_solve_one_double
  end interface

  interface ma97_solve_fredholm
      module procedure MA97_solve_fredholm_double
   end interface ma97_solve_fredholm

  interface MA97_enquire_posdef
    module procedure MA97_enquire_posdef_double
  end interface

  interface MA97_enquire_indef
    module procedure MA97_enquire_indef_double
  end interface

  interface MA97_alter
    module procedure MA97_alter_double
  end interface

  interface ma97_sparse_fwd_solve
      module procedure ma97_sparse_fwd_solve_double
   end interface ma97_sparse_fwd_solve

  interface MA97_finalise
      module procedure MA97_finalise_double
  end interface

  type MA97_control ! The scalar control of this type controls the action
    logical :: action = .true. ! pos_def = .false. only.
    real(wp) :: consist_tol = epsilon(one) 
    integer(long) :: factor_min = 20000000
    integer(short) :: nemin = nemin_default
    real(wp) :: multiplier = 1.1 
    integer :: ordering = 5 ! controls choice of ordering
    integer(short) :: print_level = 0 ! Controls diagnostic printing
    integer :: scaling = 0 ! controls use of scaling. 
    real(wp) :: small = tiny(one) ! Minimum pivot size
    real(wp) :: u = 0.01 ! Initial relative pivot threshold
    integer(short) :: unit_diagnostics = 6 ! unit for diagnostic printing.
    integer(short) :: unit_error = 6 ! unit number for error messages
    integer(short) :: unit_warning = 6 ! unit number for warning messages
    integer(long) :: min_subtree_work = 1e5 ! Minimum amount of work
    integer :: min_ldsrk_work = 1e4 ! Minimum amount of work to aim
  end type MA97_control

  type MA97_info ! The scalar info of this type returns information to user.
    integer(short) :: flag = 0     ! Value zero after successful entry.
    integer(short) :: flag68 = 0 ! error flag from hsl_mc68
    integer(short) :: flag77 = 0 ! error flag from mc77
    integer(short) :: matrix_dup = 0 ! Number of duplicated entries.
    integer(short) :: matrix_rank = 0 ! Rank of factorized matrix.
    integer(short) :: matrix_outrange = 0 ! Number of out-of-range entries.
    integer(short) :: matrix_missing_diag = 0 ! Number of missing diag. entries
    integer(short) :: maxdepth = 0 ! Maximum depth of the tree.
    integer(short) :: maxfront = 0 ! Maximum front size.
    integer(long)  :: num_factor = 0_long ! Number of entries in the factor.
    integer(long)  :: num_flops = 0_long ! Number of flops to calculate L.
    integer(short) :: num_delay = 0 ! Number of delayed eliminations.
    integer(short) :: num_neg = 0 ! Number of negative eigenvalues.
    integer(short) :: num_sup = 0 ! Number of supervariables. 
    integer(short) :: num_two = 0 ! Number of 2x2 pivots.
    integer :: ordering = 0 ! ordering actually used
    integer(short) :: stat = 0 ! STAT value (when available).
   end type MA97_info

  type MA97_akeep ! proper version has many componets!
    integer :: n
  end type MA97_akeep

  type MA97_fkeep ! proper version has many componets!
    integer :: n
  end type MA97_fkeep

contains

  subroutine MA97_analyse_double(check, n, ptr, row, akeep,                   &
                                 control, info, order)
   logical, intent(in) :: check
   integer, intent(in) :: n
   integer, intent(in) :: row(:), ptr(:)
   type (MA97_akeep), intent (out) :: akeep
   type (MA97_control), intent (in) :: control
   type (MA97_info), intent (out) :: info
   integer(short), OPTIONAL, intent (inout) :: order(:)

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_analyse with its HSL namesake ', /,        &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_analyse_double

  subroutine MA97_analyse_coord_double( n, ne, row, col, akeep,                &
                                        control, info, order)
   integer, intent(in) :: n, ne
   integer, intent(in) :: row(:), col(:)
   type (MA97_akeep), intent (out) :: akeep
   type (MA97_control), intent(in) :: control
   type (MA97_info), intent(out) :: info
   integer(short), OPTIONAL, intent (inout) :: order(:)

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_analyse_coord with its HSL namesake ', /,  &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_analyse_coord_double

  subroutine MA97_factor_double(matrix_type,val,akeep,fkeep,control,info,      &
                                scale,ptr,row)
   real(wp), intent(in) :: val(*)
   type (MA97_akeep), intent (in) :: akeep
   type (MA97_fkeep), intent (out) :: fkeep
   type (MA97_control), intent (in) :: control
   type (MA97_info), intent (inout) :: info
   real(wp), intent(inout), optional :: scale(:)
   integer(short), intent(in), optional :: ptr(akeep%n+1)
   integer(short), intent(in), optional :: row(*)

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_factorize with its HSL namesake ', /,      &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_factor_double

  subroutine MA97_factor_solve_double(matrix_type,val,nrhs,x,lx,akeep,fkeep,   &
                                      control,info,scale,ptr,row)
   real(wp), intent(in) :: val(*)
   integer(short) :: lx, nrhs
   real(wp), intent(inout) :: x(lx,nrhs)
   type (MA97_akeep), intent (in) :: akeep
   type (MA97_fkeep), intent (out) :: fkeep
   type (MA97_control), intent (in) :: control
   type (MA97_info), intent (inout) :: info
   real(wp), intent(inout), optional :: scale(:) 
   integer(short), intent(in), optional :: ptr(akeep%n+1)
   integer(short), intent(in), optional :: row(*)

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_factor_solve with its HSL namesake ', /,   &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_factor_solve_double

  subroutine MA97_factor_solve_one_double(matrix_type,val,x1,akeep,fkeep,      &
                                          control,info,scale,ptr,row)
   integer(short), intent(in) :: matrix_type 
   real(wp), intent(in) :: val(*)
   real(wp), intent(inout) :: x1(:) 
   type (MA97_akeep), intent (in) :: akeep
   type (MA97_fkeep), intent(out) :: fkeep
   type (MA97_control), intent (in) :: control
   type (MA97_info), intent (inout) :: info
   real(wp), intent(inout), optional :: scale(:)
   integer(short), intent(in), optional :: ptr(akeep%n+1)
   integer(short), intent(in), optional :: row(*)

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_factor_solve_one with its HSL namesake', /,&
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_factor_solve_one_double

  subroutine MA97_solve_double(nrhs,x,lx,akeep,fkeep,control,info,scale,job)
   integer(short), intent (in) :: nrhs, lx
   real(wp), intent (inout) :: x(lx,nrhs)
   type (MA97_akeep), intent (in) :: akeep
   type (MA97_fkeep), intent (in) :: fkeep
   type (MA97_control), intent (in) :: control
   type (MA97_info), intent (inout) :: info
   real(wp), intent(in), optional :: scale(:)
   integer(short), optional, intent (in) :: job

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_solve with its HSL namesake ', /,          &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_solve_double

  subroutine MA97_solve_one_double(x,akeep,fkeep,control,info,scale,job)
   real(wp), intent (inout) :: x(:)
   type (MA97_akeep), intent (in) :: akeep
   type (MA97_fkeep), intent (in) :: fkeep
   type (MA97_control), intent (in) :: control
   type (MA97_info), intent (inout) :: info
   real(wp), intent(in), optional :: scale(:)
   integer(short), optional, intent (in) :: job

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_solve_one with its HSL namesake ', /,      &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_solve_one_double

  subroutine MA97_solve_fredholm_double( nrhs, flag_out, x, ldx,               &
                                         akeep, fkeep, control, info )
   integer, intent(in) :: nrhs
   logical, intent(out) :: flag_out(nrhs)
   integer, intent(in) :: ldx
   real(wp), dimension(ldx,2*nrhs), intent(inout) :: x
   type(ma97_akeep), intent(in) :: akeep
   type(ma97_fkeep), intent(in) :: fkeep
   type(ma97_control), intent(in) :: control
   type(ma97_info), intent(out) :: info

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_solve_fredhom with its HSL namesake',      &
  &          /, ' and dependencies. See ', /,                                  &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option
  end subroutine MA97_solve_fredholm_double

  subroutine MA97_enquire_posdef_double(akeep,fkeep,control,info,d)
    real(wp), dimension( : ), intent(out) :: d
    type (MA97_akeep), intent (in) :: akeep
    type (MA97_fkeep), intent(in) :: fkeep
    type (MA97_control), intent (inout) :: control
    type (MA97_info), intent (inout) :: info

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_enquire_posdef with its HSL namesake', /,  &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_enquire_posdef_double

  subroutine MA97_enquire_indef_double(akeep,fkeep,control,info,piv_order,d)
    integer(short), optional, intent(out) :: piv_order(:)
    real(wp), optional, intent(out) :: d(:,:)
    type (MA97_akeep), intent (in) :: akeep
    type (MA97_fkeep), intent (in) :: fkeep
    type (MA97_control), intent (inout) :: control
    type (MA97_info), intent (inout) :: info

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_enquire_indef with its HSL namesake ', /,  &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_enquire_indef_double

  subroutine MA97_alter_double(d,akeep,fkeep,control,info)
    real(wp), intent (in) :: d(:,:)
    type (MA97_akeep), intent (in) :: akeep
    type (MA97_fkeep), intent (in) :: fkeep
    type (MA97_control), intent (inout) :: control
    type (MA97_info), intent (inout) :: info

   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_alter with its HSL namesake ', /,          &
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine MA97_alter_double

  subroutine ma97_sparse_fwd_solve_double(nbi, bindex, b, order, lflag,        &
      nxi, xindex, x, akeep, fkeep, control, info)
   integer, intent(in) :: nbi
   integer, intent(in) :: bindex(:)
   real(wp), intent(in) :: b(:)
   integer, intent(in) :: order(:)
   logical, intent(inout), dimension(:) :: lflag
   integer, intent(out) :: nxi
   integer, intent(out) :: xindex(:)
   real(wp), intent(inout) :: x(:)
   type(ma97_akeep), intent(in) :: akeep
   type(ma97_fkeep), intent(in) :: fkeep
   type(ma97_control), intent(in) :: control
   type(ma97_info), intent(out) :: info
   IF ( control%unit_error >= 0 .AND. control%print_level > 0 )                &
     WRITE( control%unit_error,                                                &
         "( ' We regret that the solution options that you have ', /,          &
  &         ' chosen are not all freely available with GALAHAD.', /,           &
  &         ' If you have HSL (formerly the Harwell Subroutine', /,            &
  &         ' Library), this option may be enabled by replacing the ', /,      &
  &         ' dummy subroutine MA97_sparse_fwd_solve with its HSL namesake ',/,&
  &         ' and dependencies. See ', /,                                      &
  &         '   $GALAHAD/src/makedefs/packages for details.' )" )
   info%flag = GALAHAD_unavailable_option

  end subroutine ma97_sparse_fwd_solve_double

  subroutine MA97_finalise_double(akeep,fkeep)
    type (MA97_akeep), intent (inout) :: akeep
    type (MA97_fkeep), intent (inout) :: fkeep
  end subroutine MA97_finalise_double

end module hsl_MA97_double
